package jp.gr.java_conf.skrb.game.trivia.client;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;

public class ClientCommunicator {
    private static final int PORT = 9000;

    private TriviaClient client;
    private Socket socket;

    private DataOutputStream ostream;

    private Thread readThread;
    private DataInputStream istream;

    public ClientCommunicator(TriviaClient client, String name, String address) 
                                                      throws IOException {
        this.client = client;

        connect(name, address);
        initReadThread();
    }

    private void connect(String name, String address) throws IOException {
        socket = new Socket(address, PORT);
        ostream = new DataOutputStream(socket.getOutputStream());
        istream = new DataInputStream(socket.getInputStream());
        
        ostream.writeUTF(name);
    }

    private void initReadThread() {
        Runnable runnable = new Runnable() {
                public void run() {
                    try {
                        Thread current = Thread.currentThread();
                        while (current == readThread) {
                            int reset = istream.readInt();
                            if (reset == -1) {
                                reset();
                            }
                        }
                    } catch (IOException ex) {
                        close();
                    }
                }
            };

        readThread = new Thread(runnable);
        readThread.start();
    }

    public void sendCounter(int count) {
        try {
            ostream.writeInt(count);
        } catch (IOException ex) {
            ex.printStackTrace();
            close();
        }
    }

    private void reset() {
        client.reset();
    }

    private void close() {
        try {
            ostream.close();
            istream.close();
            socket.close();
        } catch(IOException ex) {
            System.exit(1);
        }

        client.close();
    }
}
