package jp.gr.java_conf.skrb.game.trivia.client;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import jp.gr.java_conf.skrb.game.trivia.util.Counter;
import jp.gr.java_conf.skrb.game.trivia.util.CounterPanel;

public class TriviaClient {
    private Counter counter;
    private ClientCommunicator communicator;
    private ClientView view;

    private Properties prop;
    private String name = "";
    private String address = "";

    public TriviaClient() {
        configure();

        init();
    }

    public TriviaClient(String name, String address) {
        this.name = name;
        this.address = address;
        
        init();
    }

    public void counterIncremented(int count) {
        communicator.sendCounter(count);
    }

    public void reset() {
        counter.reset();
        view.setEnabled(true);
        view.updateView();
    }

    public void close() {
        communicator = null;

        if (Configurator.confirmConnection()) {
            System.exit(0);
        } else {
            configure();
            init();
        }
    }

    private void configure() {
        Configurator configurator = new Configurator();
        configurator.configure();
        name = configurator.getName();
        address = configurator.getAddress();
    }

    private void init() {
        counter = new Counter();
        initProperties();
        initGUI();
        
        while (true) {
            if (initCommunicator(name, address)) {
                break;
            }

            configure();
        }
    }

    private void initProperties() {
        prop = new Properties();

        try {
            FileInputStream stream = new FileInputStream("client.properties");
            prop.loadFromXML(stream);
            stream.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private boolean initCommunicator(String name, String address) {
        try {
            communicator = new ClientCommunicator(this, name, address);
        } catch (IOException ex) {
            communicator = null;
            return false;
        }

        return true;
    }

    private void initGUI() {
        if (view == null) {
            view = new DefaultClientView(this, name, counter, prop);
        }
        view.setEnabled(false);
    }

    public static void main(String[] args) {
        if (args.length == 2) {
            new TriviaClient(args[0], args[1]);
        } else {
            new TriviaClient();
        }
    }
}
